/**
 * Analyses searchbar forms and initializes widgets found inside.
 *
 * The `.presentationUiSearchbar()` method creates searchbar onto a html-template, parsing the included widgets.
 *
 * ### Examples
 *
 *    <div data-esf="ep-searchbar-root" class="ep-searchbar-root">
 *      <div class="ep-searchbar-quicksearch-wrapper">
 *        <form action="?ViewAction=MBO-SearchOrders&ObjectID=#Shop.ID" method="post" class="ep-searchbar-quicksearch" data-id="ep-searchbar-quicksearch" data-esf="ep-searchbar-quicksearch">
 *          <input type="hidden" name="OrderBy" value="#IF(#INPUT.OrderBy)#INPUT.OrderBy#ELSE#SPACE[0]CreationDate#ENDIF">
 *          <input type="hidden" name="OrderDesc" value="#IF(#DEFINED(#INPUT.OrderDesc) AND (#INPUT.OrderDesc EQ "1" OR #INPUT.OrderDesc EQ "0"))#INPUT.OrderDesc#ELSE#SPACE[0]1#ENDIF">
 *          <input type="hidden" name="ErrorAction" value="MBO-ViewProducts">
 *          <input type="hidden" name="Search" value="MBO-AllStrings">
 *
 *          <input data-esf="ep-searchbar-quicksearchinput" placeholder="{SearchProducts }" value="#IF(#DEFINED(#INPUT.WildSearchString))#INPUT.WildSearchString#ENDIF" name="WildSearchString" class="ep-ui-smartsearch ep-js ep-searchbar-quicksearchinput" data-js=".uiInput()" autocomplete="off">
 *          <button data-esf="ep-searchbar-quicksearchbutton"  class="ep-button ep-button-fake fa fa-search fa-lg ep-searchbar-quicksearch-button" title="{StartSearch }"></button>
 *        </form>
 *
 *        <button data-id="ep-searchbar-toggle" data-esf="ep-searchbar-toggle" data-ep-searchbar='{"type": "toggleSearch", "options": {"quicksearch": "ep-searchbar-quicksearch", "advancedsearch": "ep-searchbar-advancedsearch", "storageId": "searchbarProductManagerToggle"}}' type="button" class="ep-button ep-button-searchbar-toggle ep-searchbar-toggle" title="Erweiterte Suche">{ShowExtendedSearchUnity } <i class="fa fa-chevron-down"></i></button>
 *      </div>
 *
 *      <form action="?ViewAction=MBO-SearchOrders&ObjectID=#Shop.ID" method="post" data-esf="ep-searchbar-advancedsearch" data-id="ep-searchbar-advancedsearch" class="ep-searchbar-advancedsearch ui-helper-clearfix container-fluid BottomWideMargin">
 *        #IF(#INPUT.OrderBy)
 *          <input type="hidden" name="OrderBy" value="#INPUT.OrderBy">
 *        #ENDIF
 *        #IF(#INPUT.OrderDesc)
 *          <input type="hidden" name="OrderDesc" value="#INPUT.OrderDesc">
 *        #ENDIF
 *        <input type="hidden" name="ErrorAction" value="MBO-ViewOrders">
 *        <input type="hidden" name="Search" value="DetailedSearch">
 *
 *        <div class="row">
 *          // text
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for="ProductAliasString">{DocumentNo }</label>
 *            <input type="text" class="ep-js" data-js=".uiInput()" name="ProductAliasString" id="ProductAliasString" value="#IF(#DEFINED(#INPUT.Search) AND #DEFINED(#INPUT.ProductAliasString) AND #INPUT.ProductAliasString NE "")#INPUT.ProductAliasString#ENDIF" />
 *          </div>
 *
 *          // text with smartsearch
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for="ProductAliasString">{DocumentNo }</label>
 *            <input type="text" data-ep-searchbar='{"type": "smartsearch", "options": {"source": "JSONSmartSearchProductAlias"}}' class="ep-ui-smartsearch ep-js" data-js=".uiInput()" name="ProductAliasString" id="ProductAliasString" value="#IF(#DEFINED(#INPUT.Search) AND #DEFINED(#INPUT.ProductAliasString) AND #INPUT.ProductAliasString NE "")#INPUT.ProductAliasString#ENDIF" />
 *          </div>
 *
 *          // select
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for="ShippingMethodSelector">{ShippingMethod }</label>
 *            <select class="ep-language-select ep-js" name="ShippingMethodSelector" id="ShippingMethodSelector">
 *              <option value="">{AllEntries }</option>
 *              #LOOP(#Shop.ShippingMethods)
 *                <option value="#ID"#IF(#DEFINED(#INPUT.ShippingMethodSelector) AND #INPUT.ShippingMethodSelector EQ #ID) selected="selected"#ENDIF>#NameOrAlias</option>
 *              #ENDLOOP
 *            </select>
 *          </div>
 *
 *          //multiselect
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for>{Status }</label>
 *            <div data-ep-searchbar='{"type": "multiselect", "options": {}, "optionsSource": "statusSelect"}'></div>
 *          </div>
 *
 *          // checkbox
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for="IsInEditingMode">{InEditMode }</label>
 *            <input type="checkbox" name="IsInEditingMode" id="IsInEditingMode"#IF(#DEFINED(#INPUT.Search) AND #DEFINED(#INPUT.IsInEditingMode) AND #INPUT.IsInEditingMode NE "") checked="checked"#ENDIF>
 *          </div>
 *
 *          // doubledatePicker
 *          <div class="col-xs-4 BottomWideMargin">
 *            <label for="CreationDateFrom">{TimeFrame }</label>
 *            <span class="NoWrap">
 *              <input class="IdentifierHalfSize#IF(#FormError_CreationDateFrom) DialogError#ENDIF ep-js MaxWidth200" data-ep-searchbar='{"type": "datepicker", "options": {"type":"date","format":"d","time":false}}' type="text" selector="datetime" id="CreationDateFrom" name="CreationDateFrom" value="#IF(#DEFINED(#INPUT.Search) AND #DEFINED(#INPUT.CreationDateFrom) AND #INPUT.CreationDateFrom NE "")#INPUT.CreationDateFrom[datetime]#ENDIF" />
 *              <label unselectable="unselectable"> - </label>
 *            </span>
 *            <span class="NoWrap">
 *              <input class="IdentifierHalfSize#IF(#FormError_CreationDateTo) DialogError#ENDIF ep-js MaxWidth200" data-ep-searchbar='{"type": "datepicker", "options": {"type":"date","format":"d","time":false}}' type="text" selector="datetime" id="CreationDateTo" name="CreationDateTo" value="#IF(#DEFINED(#INPUT.Search) AND #DEFINED(#INPUT.CreationDateTo) AND #INPUT.CreationDateTo NE "")#INPUT.CreationDateTo[datetime]#ENDIF" />
 *            </span>
 *          </div>
 *        </div>
 *        <div class="row">
 *          <div class="col-xs-12 AlignRight">
 *            <input type="submit" data-esf="ep-searchbar-advancedSearchButton" class="ep-button ep-button-search" value="{StartSearch }">
 *          </div>
 *        </div>
 *      </form>
 *    </div>
 *
 *    <script>
 *        require([
 *          'jquery',
 *          'ep',
 *          'de_epages/presentation/ui/searchbar',
 *
 *          '$ready!'
 *        ], function($, ep, de_epages) {
 *          $('.ep-searchbar-root').presentationUiSearchbar({
 *            statusSelect: {
 *              tableHeader: '{Status }',
 *              id: 'StatusSelect',
 *              values:[{
 *                name: 'ViewedOn',
 *                value: '{ViewedOn }'
 *                #IF(#DEFINED(#INPUT.ViewedOn_checkbox)), checked: #INPUT.ViewedOn_checkbox#ENDIF
 *                #IF(#DEFINED(#INPUT.ViewedOn)), selection: #INPUT.ViewedOn#ENDIF
 *              },{
 *                name: 'CancelledOn',
 *                value: '{CancelledOn }'
 *                #IF(#DEFINED(#INPUT.CancelledOn_checkbox)), checked: #INPUT.CancelledOn_checkbox#ENDIF
 *                #IF(#DEFINED(#INPUT.CancelledOn)), selection: #INPUT.CancelledOn#ENDIF
 *              }]
 *            }
 *          });
 *        });
 *    </script>
 *
 *
 * @class jQuery.ui.presentationUiSearchbar
 * @extends jQuery.widget
 *
 * @uses jQuery
 * @uses util.storage
 * @uses de_epages.presentation.ui.multiselect
 * @uses ep.ui.datepicker
 * @uses ep.ui.smartsearch
 * @uses jQuery.ui.widget
 * @since 7.10.0
 */


/*
 * @copyright       © Copyright 2006-2017, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.presentation.ui.searchbar
 */

define( "de_epages/presentation/ui/searchbar", [
    "jquery",
    "ep/ui/smartsearch",
    "de_epages/presentation/ui/multiselect",
    "util/storage",

    "de_epages/presentation/ui/togglesection",
    "ep/ui/datepicker",
    "jquery/ui/widget"
], function ($, ep, de_epages, storage) {
    "use strict";

    $.widget('ui.presentationUiSearchbar', {

        options: {
            //startOpen: true
        },

        /**
         * Magic jquery function.
         *
         * @method _create
         * @member jQuery.ui.presentationUiSearchbar
         *
         * @since 7.10.0
         */
        _create: function() {
            var self = this;

            if (self.options.startOpen) {
                storage.localStorage(opts.options.storageId, true);
            }
            self._initWidgets();
        },

        /**
         * This method initializes wigets and functionalities (handed over by data options objects).
         *
         * @method _initWidgets
         * @member jQuery.ui.presentationUiSearchbar
         *
         * @since 7.10.0
         */
        _initWidgets: function() {
            var self = this,
                elem = self.element;

            // get all elements with a data-ep-searchbar attribute
            // this attribute contains options to initialize this element
            elem.find('[data-ep-searchbar]').each(function(index, element) {
                var $element = ep(element),
                    opts = $element.data('epSearchbar'),
                    options = opts.optionsSource ? self.options[opts.optionsSource] : opts.options;

                // init widgets depending on type
                switch (opts.type) {
                    case 'smartsearch':
                        $element.uiSmartsearch(options);
                        break;
                    case 'multiselect':
                        $element.presentationUiMultiselect(options);
                        break;
                    case 'datepicker':
                        $element.uiDatepicker(options);
                        break;
                    case 'toggleSearch':
                        self._initToggle(opts, $element);
                        break;
                    case 'toggleSection':
                        $element.presentationUiTogglesection(options);
                        break;
                }

            });
        },

        /**
         * This method toggles the visibilities of quickSearch and advancedSearch.
         *
         * @method _initToggle
         * @member jQuery.ui.presentationUiSearchbar
         *
         * @since 7.10.0
         */
        _initToggle: function(opts, $element) {
            var self = this,
                elem = self.element,
                symbol = elem.find('.ep-searchbar-toggle i.fa'),
                open = storage.localStorage(opts.options.storageId),
                $advancedSearch = elem.find('[data-id=' + opts.options.advancedsearch + ']'),
                $quickSearch = elem.find('[data-id=' + opts.options.quicksearch + ']');

            // set initial state from storage
            if (open) {
                $quickSearch.hide();
                $advancedSearch.removeClass('HideElement');
                symbol.removeClass('fa-chevron-down').addClass('fa-chevron-up');
            } else {
                $advancedSearch.addClass('HideElement');
            }

            // toggle state on click
            $element.on('click', function() {
                var open = storage.localStorage(opts.options.storageId);

                if (open) {
                    $advancedSearch.addClass('HideElement');
                    $quickSearch.show();
                    symbol.addClass('fa-chevron-down').removeClass('fa-chevron-up');
                } else {
                    $advancedSearch.removeClass('HideElement');
                    $quickSearch.hide();
                    symbol.removeClass('fa-chevron-down').addClass('fa-chevron-up');
                }

                storage.localStorage(opts.options.storageId, !open);
            });
        },

        /**
         * This method destroys the widget itself and all used widgets.
         *
         * @method destroy
         * @member jQuery.ui.presentationUiSearchbar
         *
         * @since 7.10.0
         */
        destroy: function() {
            var self = this,
                elem = self.element;

            // destroy widgets
            elem.find('[data-ep-searchbar]').each(function(index, element) {
                var $element = ep(element),
                    opts = $element.data('epSearchbar'),
                    options = opts.optionsSource ? self.options[opts.optionsSource] : opts.options;

                // init widgets depending on type
                switch (opts.type) {
                    case 'smartsearch':
                        $element.uiSmartsearch('destroy');
                        break;
                    case 'multiselect':
                        $element.presentationUiMultiselect('destroy');
                        break;
                    case 'datepicker':
                        $element.uiDatepicker('destroy');
                        break;
                }
            });

            self._superApply(arguments);
        }

    });

    return de_epages;

});
